use strict;
use Win32::SerialPort;

# 
# ,       (   10240)
#    ,    MK90.BIN

#
# Configuration
#
my $com_number = 13;        # COM-port number, run arduino IDE to get it
my $arduino_baud = 38400;   # COM-port speed, according to arduino sketch settings 
my $arduino_databits = 8;   # COM-port databits, ususally 8bit and can't be changed
my $arduino_stopbits = 1;   # COM-port stopbits, ususally 1 bit and can't be changed
#
# Defaul values
#
my $file_name = 'MK90.BIN'; # File name for read/write operation
my $CARDSIZE = 10240;

#
# Global vars
#

my $ob; # Port object reference
my $mode; 

sub version() {
  print "\nArduino SMP Reader/Writer v1.0\n";
}

sub help() {
  print "\nUsage:\t$0 mode [filename]\n\n";
  print "where:\n";
  print "\tmode:     write|read\n";
  print "\tfilename: file for reading or writing (default = MK90.BIN)\n";
}

#
# READ/VERIFY operation
#
sub read_smp($$) {
  my ($fname,$verify) = @_;
  my ($cnt,$res,$phonema);

  if ($verify) {
    open FH, "<$fname" or die "Can't open $fname for reading: $!\n";
  } else {
    open FH, ">$fname" or die "Can't open $fname for writing: $!\n";
  }
  binmode FH;

  my $retval = 0; # error counter for verify

  # send command
  $ob->write('R');  
  # wait for 'Start' from Arduino
  while ( !($phonema =~ /Start/ ) ) {
    ($cnt,$res) = $ob->read(1);
    $phonema .= $res;
  }

  my $max_bytes = $CARDSIZE;
  my $buf="";
  while ($max_bytes) {
    ($cnt,$res) = $ob->read(1); # get ONE char from buffer
    next unless $cnt;
    if ($verify) {
      read(FH, $buf, 1);
      if ( $buf != $res ) {
        $retval++;
        print STDERR "Error address ".($CARDSIZE-$max_bytes)." FILE = ".ord($buf).", SMP = ".ord($res)."\n";
      }
    } else {
      print FH $res;      
    }
    $max_bytes--;
  }

  close FH;
  return $retval;
}

#
# WRITE operation
#
sub write_smp($) {
  my ($fname) = @_;
  my ($cnt,$res,$phonema);

  open FH, "<$fname" or die "Can't open $fname for reading: $!\n";
  binmode FH;
  die "Sorry, can't write $fname: size != $CARDSIZE\n" unless (-s FH == $CARDSIZE);
  # Send command 
  $ob->write('W');
  # Wait for 'Ready' status from Arduino
  while ( !($phonema =~ /Ready/ ) ) {
    ($cnt,$res) = $ob->read(1);
    $phonema .= $res;
  }
  # wite $CARDSIZE bytes
  my $max_bytes = $CARDSIZE;
  my $buf;
  while (read(FH, $buf, 1) and $max_bytes) {
    $ob->write($buf);
    $max_bytes--;
  }
  close FH;
  # verify written data
  if (my $err_cnt = &read_smp($fname,1)) {
    print "\nVerify operation FAILED, $err_cnt errored byte(s) found. Please, try again.\n";
  } else {
    print "SMP written successefully.\n";
  }
}

##########################
#
# MAIN
#
#########################

&version();

# command line parsing 
if (scalar @ARGV < 1 or scalar @ARGV > 2) {
  &help();
  exit(0);
}

if ($ARGV[0] eq 'write') { 
  $mode = 'write';
} elsif ($ARGV[0] eq 'read') {
  $mode = 'read'; 
} else {
  die "Unknown workmode '".$ARGV[1]."'\n";
}

$file_name = $ARGV[1] if (scalar @ARGV == 2);

# opening port
my $port;
if ($com_number < 10){
 $port = "COM".$com_number;
} else {
 $port = "\\\\.\\COM".$com_number;
}

$ob = new Win32::SerialPort ($port) || die "Can't open $port: $!\n";

my $baud = $ob->baudrate;
my $parity = $ob->parity;
my $data = $ob->databits;
my $stop = $ob->stopbits;
my $hshake = $ob->handshake;

print "\nCurrent COM$com_number settings: ";
print "B = $baud, D = $data, S = $stop, P = $parity, H = $hshake\n";

$ob->buffers(10240, 10240);
$ob->baudrate($arduino_baud) || die "fail setting baudrate $arduino_baud\n";
$ob->databits($arduino_databits) || die "fail setting databits $arduino_databits\n";
$ob->stopbits($arduino_stopbits) || die "fail setting databits $arduino_stopbits\n";
$ob->parity("none") || die "fail setting parity to \"none\"\n";
$ob->write_settings() || die "fail apply COM-port settings, $!\n";

if ($mode eq 'read') {
  &read_smp($file_name,0);
} elsif ($mode eq 'write') {
  &write_smp($file_name);
} else {
  die "Unexpected state\n";
}
