/*
     /     -90  Serial-.
         by Piotr Piatek
    http://www.pisi.com.pl/piotr433/mk90prge.htm
*/

#define P_DELAY  8
#define CARDSTART 0x0000
#define CARDSIZE 0x2800
#define BAUD  38400

int selPin = 5;
int clkPin = 7;
int dataPin = 6;

int cmd = -1;
byte incChar = 0;
unsigned int writeCounter = 0;

void prompt() {
  Serial.println("Command ('H' - for help)>");
}

void setup() {
  pinMode(selPin, OUTPUT);    // SELECT -     
  pinMode(clkPin, OUTPUT);    // CLOCK  -  Master  Arduino
  pinMode(dataPin, OUTPUT);   // DATA   -   
  
  digitalWrite(selPin, HIGH); 
  digitalWrite(clkPin, HIGH); 
  digitalWrite(dataPin, HIGH); 
  delay(550);
  digitalWrite(selPin, LOW); 
  delay(110);
  digitalWrite(dataPin, LOW); 
  delay(110);
  digitalWrite(dataPin, HIGH); 
  delay(110);
  digitalWrite(dataPin, LOW); 
  delay(110);
  digitalWrite(dataPin, HIGH); 
  delay(110);
  digitalWrite(selPin, HIGH); 
  delay(110);
  
  Serial.begin(BAUD);       //     serial
  prompt();  
  digitalWrite(13, HIGH);  //  
}

void help() {
  Serial.println("Commands:");
  Serial.println("  R - read from SMP to serial");
  Serial.println("  W - write from serial to SMP");  
  Serial.println("  F - test: flash diode at pin 13");
}

byte readByte() {
  byte x = 0;
  for (byte mask=0x80; mask!=0; mask>>=1)
  {
    digitalWrite(clkPin, LOW);
    delayMicroseconds(P_DELAY);
    digitalWrite(clkPin, HIGH);
    delayMicroseconds(P_DELAY);
    if (digitalRead(dataPin)) x |= mask;
  }
  return x;
}

void sendByte(byte b) {
  for (byte mask=0x80; mask!=0; mask>>=1)
  {
    digitalWrite(dataPin, ( (b & mask) != 0) ? HIGH : LOW);
    digitalWrite(clkPin, HIGH);
    delayMicroseconds(P_DELAY);
    digitalWrite(clkPin, LOW);
    delayMicroseconds(P_DELAY);
  }
}

void sendAddress(unsigned int addr) {
  digitalWrite(dataPin,LOW);  // DATA=0
  delayMicroseconds(P_DELAY);
  digitalWrite(selPin,LOW);   // SELECT=0R
  delayMicroseconds(P_DELAY);
  sendByte (0xA0);            // WriteAddress cmd = 0xA0
  sendByte (addr/256);
  sendByte (addr%256);
//  digitalWrite(dataPin,LOW);  // DATA=0
//  digitalWrite(clkPin,LOW);   // CLOCK=0
//  delayMicroseconds(P_DELAY);
//  digitalWrite(clkPin,HIGH);   // CLOCK=1
//  delayMicroseconds(P_DELAY);
  digitalWrite(selPin,HIGH);   // SELECT=1
  delayMicroseconds(P_DELAY);
  digitalWrite(dataPin,HIGH);  // DATA=1
  delayMicroseconds(P_DELAY);
}

void outputArray(unsigned int cnt) {
  pinMode(dataPin, OUTPUT);   // Write Only
  sendAddress(CARDSTART);             // Setup Address
  digitalWrite(dataPin,LOW);  // ClrBit (DATA);		/* DATA=0 */
  delayMicroseconds(P_DELAY);
  digitalWrite(selPin,LOW);   // ClrBit (SELECT);	/* SELECT=0 */
  delayMicroseconds(P_DELAY);
  sendByte (0xD0);            // ReadPostincrement cmd = 0xd0
  //digitalWrite(dataPin,HIGH); // ?? SetBit (DATA);	/* DATA=input */
  digitalWrite(clkPin,HIGH);  //    SetBit (CLOCK);	/* CLOCK=1 */
  delayMicroseconds(P_DELAY);

  pinMode(dataPin, INPUT);  
  for (unsigned int i=0; i<cnt; i++) {
    Serial.print(readByte(),BYTE);
  }
  Serial.println();
  
  digitalWrite(selPin,HIGH);  
  delayMicroseconds(P_DELAY);
}

void inputArray(unsigned int cnt) {
  pinMode(dataPin, OUTPUT);   // Write Only
  sendAddress(CARDSTART);             // Setup Address
  digitalWrite(dataPin,LOW);  // ClrBit (DATA);		/* DATA=0 */
  delayMicroseconds(P_DELAY);
  digitalWrite(selPin,LOW);   // ClrBit (SELECT);	/* SELECT=0 */
  delayMicroseconds(P_DELAY);  
  sendByte (0xC0);            // WritePostincrement cmd = 0xc0
  //digitalWrite(dataPin,HIGH); // ?? SetBit (DATA);	/* DATA=input */
  digitalWrite(clkPin,HIGH);  //    SetBit (CLOCK);	/* CLOCK=1 */
  delayMicroseconds(P_DELAY);
  
  writeCounter = CARDSIZE; 
  while (writeCounter) {
    if (Serial.available() > 0) {
      incChar = Serial.read();
      sendByte(incChar);
      writeCounter--;
    }
  }
  digitalWrite(selPin,HIGH);  
  delayMicroseconds(P_DELAY);  
}

void loop() {
  
  cmd = Serial.read();
  
  if (cmd != -1) {
    if (cmd == 'H') {
      help();
    }
    if (cmd == 'R') {
      Serial.print("Start");
      outputArray(CARDSIZE);
    }
    if (cmd == 'W') {
      Serial.flush(); // Kill any data in receive buffer
      Serial.print("Ready");
      inputArray(CARDSIZE);
    }
    if (cmd == 'F') {
      digitalWrite(13, HIGH);
      delay(200);
      digitalWrite(13, LOW);
      delay(200);
      digitalWrite(13, HIGH);      
    }
    prompt();
  }  
}
